#!/usr/bin/env python
import sys
import getopt
import shutil
import os

class ClientInstaller:
   def __init__(self):
      self.bin_dir = None
      self.data_dir = None
      self.man_dir = None
      self.img_dir = None
      self.menu_dir = None
      self.client_data = ['juke_client.py', 'control_handlers.py', 'navigation_handlers.py',\
'senders.py', 'connect_handlers.py', 'xml_parsers.py', 'juke_socket.py', 'command_stores.py',\
'dialogs.py', 'juke_client.xml', 'juke_config.xml', 'xml_editors.py', 'connection_utilities.py',\
'edit_conn_handlers.py']
      self.client_imgs = ['juketv_logo_24x24_transparent.png', 'juketv_logo_200x200_transparent.png', 'juketv_logo_48x48_transparent.png']
      self.client_man = 'jukeclient.1'
      self.exename = 'jukeclient'
   
   def print_usage(self, arg0):
      print 'Proper Usage:'
      print arg0 + ' [--help] | [ --bin <binary_dir> --data <data_dir> --man <man_dir> --img <img_dir> --menu <menu_dir> ]'
   
   def set_args(self, opts, progname):
      for opt, arg in opts:
         if opt == '-h' or opt == '--help':
            self.print_usage(progname)
            sys.exit(0)
         elif opt == '-b' or opt == '--bin':
            self.bin_dir = self.remove_trailing_slash(arg)
         elif opt == '-d' or opt == '--data':
            self.data_dir = self.remove_trailing_slash(arg)
         elif opt == '-m' or opt == '--man':
            self.man_dir = self.remove_trailing_slash(arg)
         elif opt == '-i' or opt == '--img':
            self.img_dir = self.remove_trailing_slash(arg)
         elif opt == '-u' or opt == '--menu':
            self.menu_dir = self.remove_trailing_slash(arg)

   def remove_trailing_slash(self, text):
      if text[-1] == '/':
         return text[:-1]
      return text
   
   def check_args(self, arg0):
      missing_params = False

      if self.bin_dir == None:
         print 'Missing bin_dir.'
         missing_params = True
      if self.data_dir == None:
         print 'Missing data_dir.'
         missing_params = True
      if self.man_dir == None:
         print 'Missing man_dir.'
         missing_params = True
      if self.img_dir == None:
         print 'Missing img_dir.'
         missing_params = True
      if self.menu_dir == None:
         print 'Missing menu_dir.'
         missing_params = True

      if missing_params:
         self.print_usage(arg0)
         sys.exit(1)
   
   def install(self):
      self.copy_data()
      self.copy_imgs()
      self.copy_man()
      self.write_main_binary()
      self.write_desktop_entry()
      self.set_file_permissions()

   def copy_data(self):
      if not os.path.exists(self.data_dir):
         os.makedirs(self.data_dir)
      for source in self.client_data:
         file = 'src/' + source
         shutil.copyfile(file, self.data_dir + '/' + source)
      for img in self.client_imgs:
         file = 'imgs/' + img
         shutil.copyfile(file, self.data_dir + '/' + img)

   def copy_imgs(self):
      if not os.path.exists(self.img_dir):
         os.makedirs(self.img_dir)
      for img in self.client_imgs:
         file = 'imgs/' + img
         shutil.copyfile(file, self.img_dir + '/' + img)

   def copy_man(self):
      if not os.path.exists(self.man_dir):
         os.makedirs(self.man_dir)
      shutil.copyfile('man/' + self.client_man, self.man_dir + '/' + self.client_man)

   def write_main_binary(self):
      main_bin = self.get_contents()
      if not os.path.exists(self.bin_dir):
         os.makedirs(self.bin_dir)
      file = open(self.bin_dir + '/' + self.exename, 'w')
      file.write(main_bin)
      file.close()

   def write_desktop_entry(self):
      entry = self.get_desktop_entry_contents()
      if not os.path.exists(self.menu_dir):
         os.makedirs(self.menu_dir)
      file = open(self.menu_dir + '/jukeclient.desktop', 'w')
      file.write(entry)
      file.close()

   def set_file_permissions(self):
      # TODO: i want to use the python version of chmod, but i can't
      # seem to get it to work
      try:
         os.system('chmod 755 "' + self.bin_dir + '/' + self.exename + '"')
      except:
         pass
      try:
         os.chmod(self.bin_dir + '/' + self.exename, stat.S_IXUSR | stat.S_IXOTH | stat.S_IXGRP)
      except:
         pass

   def get_contents(self):
      contents = '''#!/usr/bin/env python
import os
os.chdir("''' + self.data_dir + '''")
os.system("python juke_client.py")'''
      return contents
   
   def main(self, argv):
      try:
         pair, leftovers = getopt.getopt(argv[1:], 'hb:d:m:i:u:', ['help', 'bin=', 'data=', 'man=', 'img=', 'menu='])
      except:
         print 'Invalid arguments.'
         self.print_usage(argv[0])
         sys.exit(1)
      self.set_args(pair, argv[0])
      self.check_args(argv[0])
      self.install()

   def get_desktop_entry_contents(self):
      contents = '''[Desktop Entry]
Name=JukeTV
Comment=The Video JukeBox
Exec=''' + self.bin_dir + '/' + self.exename + '''
Icon=''' + self.img_dir + '''/juketv_logo_48x48_transparent.png
Terminal=false
Type=Application
Categories=GTK;GNOME;AudioVideo;Player;Video;
StartupNotify=true'''
      return contents

if __name__ == '__main__':
   installer = ClientInstaller()
   installer.main(sys.argv)
