#!/usr/bin/env python
import sys
import getopt
import shutil
import os

class ServerInstaller:
   def __init__(self):
      self.bin_dir = None
      self.data_dir = None
      self.man_dir = None
      self.server_data = ['net_listener.py', 'xml_parsers.py', 'juke_socket.py', 'server_message_parser.py',\
'commands.py', 'path_converter.py', 'command_stores.py', 'juke_config.xml']
      self.server_man = 'jukeserver.1'
      self.exename = 'jukeserver'
   
   def print_usage(self, arg0):
      print 'Proper Usage:'
      print arg0 + ' [--help] | [ --bin <binary_dir> --data <data_dir> --man <man_dir> ]'
   
   def set_args(self, opts, progname):
      for opt, arg in opts:
         if opt == '-h' or opt == '--help':
            self.print_usage(progname)
            sys.exit(0)
         elif opt == '-b' or opt == '--bin':
            self.bin_dir = arg
         elif opt == '-d' or opt == '--data':
            self.data_dir = arg
         elif opt == '-m' or opt == '--man':
            self.man_dir = arg
   
   def check_args(self, arg0):
      missing_params = False

      if self.bin_dir == None:
         print 'Missing bin_dir'
         missing_params = True
      if self.data_dir == None:
         print 'Missing data_dir'
         missing_params = True
      if self.man_dir == None:
         print 'Missing man_dir'
         missing_params = True

      if missing_params:
         self.print_usage(arg0)
         sys.exit(1)

   def install(self):
      if not os.path.exists(self.data_dir):
         os.makedirs(self.data_dir)
      for source in self.server_data:
         file = 'src/' + source
         shutil.copyfile(file, self.data_dir + '/' + source)
      if not os.path.exists(self.man_dir):
         os.makedirs(self.man_dir)
      shutil.copyfile('man/' + self.server_man, self.man_dir + '/' + self.server_man)
      main_bin = self.get_contents()
      if not os.path.exists(self.bin_dir):
         os.makedirs(self.bin_dir)
      file = open(self.bin_dir + '/' + self.exename, 'w')
      file.write(main_bin)
      file.close()
      # TODO: i want to use the python version of chmod, but i can't
      # seem to get it to work
      try:
         os.system('chmod 755 ' + self.bin_dir + '/' + self.exename)
      except:
         pass
      try:
         os.chmod(self.bin_dir + '/' + self.exename, stat.S_IXUSR | stat.S_IXOTH | stat.S_IXGRP)
      except:
         pass

   def get_contents(self):
      contents = '''#!/usr/bin/env python
import os
os.chdir("''' + self.data_dir + '''")
os.system("python net_listener.py")'''
      return contents

   def main(self, argv):
      try:
         pair, leftovers = getopt.getopt(argv[1:], 'hb:d:m:', ['help', 'bin=', 'data=', 'man='])
      except:
         self.print_usage(argv[0])
         sys.exit(1)
      self.set_args(pair, argv[0])
      self.check_args(argv[0])
      self.install()

if __name__ == '__main__':
   installer = ServerInstaller()
   installer.main(sys.argv)
