import os
from sys import exit
from path_converter import PathConverter, replaceStr

class Command:
   def __init__(self):
      self.command = None

   def execute(self):
      if self.command != None:
         childpid = os.fork()
         if childpid:
            return self.buildReply(childpid)
         else:
            # execute command and exit with its return code
            exit(os.system(self.command))

   def buildReply(self, pid):
      return "<reply type='pid'><pid>" + str(pid) + "</pid></reply>"

class PlayCommand(Command):
   def __init__(self, filename, commandFormat):
      Command.__init__(self)
      self.command = replaceStr(commandFormat, '%f', filename)

class PauseCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class ResumeCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class SkipFwdCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class SkipBwdCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class VolUpCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class VolDownCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command

class LsCommand(Command):
   def __init__(self, dirList, virtualDir):
      self.converter = PathConverter(dirList)
      self.virtualDir = virtualDir

   def execute(self):
      if self.virtualDir == '/':
         return self.buildRootReply()
      else:
         try:
            absolutePath = self.converter.virtualToAbsolute(self.virtualDir)
            results = os.listdir(absolutePath)
         except:
            results = ''
         return self.buildNonRootReply(results, absolutePath)

   def buildRootReply(self):
      reply = "<reply type='ls'><filelist>"
      for dir in self.converter.dirList:
         reply += '<directory>' + dir[0] + '</directory>'
      reply += '</filelist></reply>'
      return reply

   def buildNonRootReply(self, results, absolutePath):
      if not absolutePath.endswith('/'):
         absolutePath += '/'
      reply = "<reply type='ls'><filelist>"
      for file in results:
         if os.path.isdir(absolutePath + '/' + file):
            reply += '<directory>' + file + '</directory>'
         else:
            reply += '<file>' + file + '</file>'
      reply += '</filelist></reply>'
      return reply

class ExitCommand(Command):
   def __init__(self, command):
      Command.__init__(self)
      self.command = command
