from senders import SenderException
import dialogs
from xml_parsers import ClientParser
from xml_editors import ClientEditor
#import gtk
import connection_utilities

class ConnectDialogHandler:
   def __init__(self, builder, handlers, navHandler):
      self.builder = builder
      self.handlers = handlers
      self.navHandler = navHandler
      self.initConnText()
      connection_utilities.initAnyConnView('connView', self.builder, self.onConnectionSelectionChanged)
      self.selectedHost = ''
      self.selectedPort = ''
      # bug where glade cannot set default value
      # for spin buttons
      #self.setSpinButtonDefault()

   def initConnText(self):
      parser = ClientParser()
      parser.load()
      hostEntry = self.builder.get_object('hostEntry')
      hostEntry.set_text(parser.getLastHost())
      portSpinButton = self.builder.get_object('portSpinButton')
      try:
         port = int(parser.getLastPort())
      except:
         port = 51787
      portSpinButton.set_value(port)


   #def initConnView(self):
   #   connView = self.builder.get_object('connView')
   #   connView.get_selection().connect('changed', self.onConnectionSelectionChanged, None)
   #   self.appendColumn(connView, 'Host', 0)
   #   self.appendColumn(connView, 'Port', 1)

   def writeLastConnection(self, host, port):
      editor = ClientEditor()
      editor.load()
      editor.writeLastConnection(host, port)

   def onHostEntryEnterPressed(self, entry):
      self.onConnectConnectButtonClicked(self.builder.get_object('connectConnectButton'))

   def onPortSpinButtonEnterPressed(self, spinbutton):
      self.onConnectConnectButtonClicked(self.builder.get_object('connectConnectButton'))

   def onConnectionSelectionChanged(self, treeSelection, userdata):
      connStore, iter = treeSelection.get_selected()
      self.selectedHost = connStore.get(iter, 0)[0]
      self.selectedPort = connStore.get(iter, 1)[0]

   #def appendColumn(self, treeview, text, index):
   #   column = gtk.TreeViewColumn(text)
   #   coltext = gtk.CellRendererText()
   #   column.pack_start(coltext, True)
   #   column.add_attribute(coltext, 'text', index)
   #   treeview.append_column(column)

   #def setSpinButtonDefault(self):
   #   button = self.builder.get_object('portSpinButton')
   #   button.set_value(51787)

   def onConnectConnectButtonClicked(self, button):
      # TODO: handle connection errors
      dialog = self.builder.get_object('connectDialog')
      address = self.getAddress()
      port = self.getPort()
      for handler in self.handlers:
         handler.setAddress(address)
         handler.setPort(port)
      try:
         self.navHandler.setCurDir('/', True)
         dialog.hide()
         self.writeLastConnection(address, port)
      except SenderException:
         msg = 'Error Connecting to ' + address
         smsg = 'Make sure that the host and port are correct and that the JukeTV server is running.'
         errorDialog = dialogs.JukeErrorDialog(dialog, msg, smsg)
         errorDialog.run()

   def onConnectCancelButtonClicked(self, button):
      dialog = self.builder.get_object('connectDialog').hide()

   def onConnectOpenButtonClicked(self, button):
      parser = ClientParser()
      parser.load()
      conns = parser.getConnectionList()
      connection_utilities.populateAnyConnView(conns, 'connStore', self.builder)
      self.builder.get_object('openConnectionWindow').show()

   #def populateConnView(self, conns):
   #   connStore = self.builder.get_object('connStore')
   #   connStore.clear()
   #   for conn in conns:
   #      connStore.append([conn[0], conn[1]])

   def onConnectSaveButtonClicked(self, button):
      host = self.builder.get_object('hostEntry').get_text()
      port = self.builder.get_object('portSpinButton').get_text()
      editor = ClientEditor()
      editor.load()
      editor.addSavedConnection(host, port)
      dialog = self.builder.get_object('messageDialog')
      dialog.run()
      dialog.hide()

   def onOpenConnOpenButtonClicked(self, button):
      self.builder.get_object('hostEntry').set_text(self.selectedHost)
      self.builder.get_object('portSpinButton').set_value(int(self.selectedPort))
      self.builder.get_object('openConnectionWindow').hide()

   def onOpenConnCancelButtonClicked(self, button):
      self.builder.get_object('openConnectionWindow').hide()

   def getAddress(self):
      hostEntry = self.builder.get_object('hostEntry')
      return hostEntry.get_text()

   def getPort(self):
      portButton = self.builder.get_object('portSpinButton')
      return portButton.get_value_as_int()
