import gtk
from senders import *
import dialogs

class CommandHandler:
   def __init__(self):
      self.address = None
      self.port = None

   def setAddress(self, addr):
      self.address = addr

   def setPort(self, port):
      self.port = port

   def send(self, sender):
      try:
         sender.connect()
         sender.send()
      except SenderException as e:
         parent = self.builder.get_object('mainWindow')
         msg = 'Error Sending Command'
         errorDialog = dialogs.JukeErrorDialog(parent, msg, str(e))
         errorDialog.run()
         raise e

class VolumeHandler(CommandHandler):
   def __init__(self, builder):
      CommandHandler.__init__(self)
      self.builder = builder

   def onDownClicked(self, button):
      self.send(VolDownSender(self.address, self.port))

   def onUpClicked(self, button):
      self.send(VolUpSender(self.address, self.port))

class SkipHandler(CommandHandler):
   def __init__(self, builder):
      CommandHandler.__init__(self)
      self.builder = builder

   def onRewindClicked(self, button):
      self.send(SkipBwdSender(self.address, self.port))

   def onForwardClicked(self, button):
      self.send(SkipFwdSender(self.address, self.port))

class PlayPauseStopHandler(CommandHandler):
   def __init__(self, builder, navHandler):
      CommandHandler.__init__(self)
      self.builder = builder
      self.handler = self.play
      self.navHandler = navHandler

   def onPlayPauseClicked(self, button):
      self.handler(button)

   def onStopClicked(self, button):
      try:
         self.send(ExitSender(self.address, self.port))
         playButton = self.builder.get_object('playPauseButton')
         playButton.set_image(self.getStockButtonImage(gtk.STOCK_MEDIA_PLAY))
         self.handler = self.play
      except:
         pass

   def play(self, button):
      file = self.navHandler.getSelectedFileName()
      if file != None:
         try:
            self.send(PlaySender(file, self.address, self.port))
            self.handler = self.pause
            button.set_image(self.getStockButtonImage(gtk.STOCK_MEDIA_PAUSE))
         except:
            pass

   def pause(self, button):
      try:
         self.send(PauseSender(self.address, self.port))
         self.handler = self.resume
         button.set_image(self.getStockButtonImage(gtk.STOCK_MEDIA_PLAY))
      except:
         pass

   def resume(self, button):
      try:
         self.send(ResumeSender(self.address, self.port))
         self.handler = self.pause
         button.set_image(self.getStockButtonImage(gtk.STOCK_MEDIA_PAUSE))
      except:
         pass

   def getStockButtonImage(self, stockid):
      image = gtk.Image()
      image.set_from_stock(stockid, gtk.ICON_SIZE_BUTTON)
      return image
