#!/usr/bin/env python

import pygtk
pygtk.require('2.0')
import gtk
from control_handlers import *
from navigation_handlers import NavigationHandler
from senders import SenderException
from connect_handlers import ConnectDialogHandler
from edit_conn_handlers import EditConnectionHandler

class JukeClient:
   def __init__(self, bfile):
      self.builder = gtk.Builder()
      self.builder.add_from_file(bfile)
      self.setSignalHandlers()
      mainWindow = self.builder.get_object('mainWindow')
      mainWindow.show()

   def setSignalHandlers(self):
      handlers = []
      navHandler = NavigationHandler(self.builder)
      handlers.append(navHandler)
      volCallbacks = self.getVolCallbacks(handlers)
      skipCallbacks = self.getSkipCallbacks(handlers)
      playPauseStopCallbacks = self.getPlayPauseStopCallbacks(navHandler, handlers)
      connHandler = ConnectDialogHandler(self.builder, handlers, navHandler)
      editConnHandler = EditConnectionHandler(self.builder)
      self.builder.connect_signals( {'onMainWindowDestroy' : self.destroy,\
                                 'onStopClicked' : playPauseStopCallbacks[1],\
                                 'onRewindClicked' : skipCallbacks[0],\
                                 'onPlayPausedClicked' : playPauseStopCallbacks[0],\
                                 'onForwardClicked' : skipCallbacks[1],\
                                 'onVolDownClicked': volCallbacks[0],\
                                 'onVolUpClicked' : volCallbacks[1],\
                                 'onConnectButtonClicked' : self.onConnectButtonClicked,\
                                 'onConnectConnectButtonClicked' : connHandler.onConnectConnectButtonClicked,\
                                 'onConnectCancelButtonClicked' : connHandler.onConnectCancelButtonClicked,\
                                 'onConnectOpenButtonClicked' : connHandler.onConnectOpenButtonClicked,\
                                 'onConnectSaveButtonClicked' : connHandler.onConnectSaveButtonClicked,\
                                 'showAboutDialog' : self.showAboutDialog,\
                                 'onAboutDialogClose' : self.hideAboutDialog,\
                                 'onOpenConnOpenButtonClicked' : connHandler.onOpenConnOpenButtonClicked,
                                 'onOpenConnCancelButtonClicked' : connHandler.onOpenConnCancelButtonClicked,\
                                 'onHostEntryEnterPressed' : connHandler.onHostEntryEnterPressed,\
                                 'onPortSpinButtonEnterPressed' : connHandler.onPortSpinButtonEnterPressed,\
                                 'onEditConnItemClicked' : editConnHandler.onEditConnItemClicked,\
                                 'onEditConnDoneButtonClicked' : editConnHandler.onEditConnDoneButtonClicked,\
                                 'onEditConnDeleteButtonClicked' : editConnHandler.onEditConnDeleteButtonClicked,\
                                 'onEditConnAddButtonClicked' : editConnHandler.onEditConnAddButtonClicked,\
                                 'onAddSaveButtonClicked' : editConnHandler.onAddSaveButtonClicked,\
                                 'onAddCancelButtonClicked' : editConnHandler.onAddCancelButtonClicked} )

   def getVolCallbacks(self, handlers):
      handler = VolumeHandler(self.builder)
      handlers.append(handler)
      return handler.onDownClicked, handler.onUpClicked

   def getSkipCallbacks(self, handlers):
      handler = SkipHandler(self.builder)
      handlers.append(handler)
      return handler.onRewindClicked, handler.onForwardClicked

   def getPlayPauseStopCallbacks(self, navHandler, handlers):
      handler = PlayPauseStopHandler(self.builder, navHandler)
      handlers.append(handler)
      return handler.onPlayPauseClicked, handler.onStopClicked

   def onConnectButtonClicked(self, button):
      dialog = self.builder.get_object('connectDialog')
      dialog.show()

   def destroy(self, window):
      gtk.main_quit()

   def showAboutDialog(self, aboutItem):
      dialog = self.builder.get_object('aboutDialog')
      dialog.show()

   def hideAboutDialog(self, dialog, data):
      dialog.hide()

if __name__ == '__main__':
   client = JukeClient('juke_client.xml')
   gtk.main()
