import gtk
from senders import LsSender, SenderException
import dialogs

class NavigationHandler:
   def __init__(self, builder):
      self.selectedFile = None
      self.builder = builder
      self.address = None
      self.port = None
      self.initDirView()
      self.initFileView()
      # lets try not connecting on startup
      #self.setCurDir('/', True)

   def setAddress(self, addr):
      self.address = addr

   def setPort(self, port):
      self.port = port

   def onDirRowSelectionChanged(self, treeSelection, userdata):
      dirStore, iter = treeSelection.get_selected()
      dir = dirStore.get(iter, 0)[0]
      try:
         self.setRelativeDir(dir)
      except SenderException as e:
         parent = self.builder.get_object('mainWindow')
         msg = 'Error Getting Directory Contents'
         errorDialog = dialogs.JukeErrorDialog(parent, msg, str(e))
         errorDialog.run()

   def initDirView(self):
      dirView = self.builder.get_object('dirView')
      dirView.get_selection().connect('changed', self.onDirRowSelectionChanged, None)
      column = gtk.TreeViewColumn('Directory')
      coltext = gtk.CellRendererText()
      colimg = gtk.CellRendererPixbuf()
      column.pack_start(colimg, False)
      column.pack_start(coltext, True)
      column.add_attribute(coltext, 'text', 0)
      column.add_attribute(colimg, 'pixbuf', 1)
      dirView.append_column(column)

   def initFileView(self):
      fileView = self.builder.get_object('fileView')
      fileView.get_selection().connect('changed', self.onFileRowSelectionChanged, None)
      column = gtk.TreeViewColumn('Filename')
      coltext = gtk.CellRendererText()
      colimg = gtk.CellRendererPixbuf()
      column.pack_start(colimg, False)
      column.pack_start(coltext, True)
      column.add_attribute(coltext, 'text', 0)
      column.add_attribute(colimg, 'pixbuf', 1)
      fileView.append_column(column)

   def getSelectedFileName(self):
      if self.currentDir.endswith('/'):
         return self.currentDir + self.selectedFile
      else:
         return self.currentDir + '/' + self.selectedFile

   def onFileRowSelectionChanged(self, treeSelection, userdata):
      fileStore, iter = treeSelection.get_selected()
      self.selectedFile = fileStore.get(iter, 0)[0]

   def setCurDir(self, vdir, root=False):
      if self.address != None and self.port != None:
         self.curDir = vdir
         sender = LsSender(vdir, self.address, self.port)
         sender.connect()
         dirs, files = sender.send()
         self.setDirs(dirs, root)
         self.setFiles(files)
         self.setAddressBar(vdir)

   def goToParent(self):
      index = -1
      for i in range(len(self.curDir)):
         if self.curDir[i] == '/':
            index = i
      if index == 0:
         self.setCurDir('/', True)
      else:
         self.setCurDir(self.curDir[:index])

   def setRelativeDir(self, rdir):
      if self.curDir == '/':
         self.setCurDir(self.curDir + rdir)
      elif rdir == '..':
         self.goToParent()
      else:
         self.setCurDir(self.curDir + '/' + rdir)

   def setDirs(self, dirs, root=False):
      dirs.sort()
      dirStore = self.builder.get_object('dirStore')
      dirStore.clear()
      img = gtk.icon_theme_get_default().load_icon('folder', gtk.ICON_SIZE_MENU, 0)
      if not root:
         dirStore.append(['..', img])
      for dir in dirs:
         dirStore.append([dir, img])

   def setFiles(self, files):
      files.sort()
      fileStore = self.builder.get_object('fileStore')
      fileStore.clear()
      img = gtk.icon_theme_get_default().load_icon('video', gtk.ICON_SIZE_MENU, 0)
      for file in files:
         fileStore.append([file, img])

   def setAddressBar(self, dir):
      self.currentDir = dir
      self.builder.get_object('addressEntry').set_text('Location:   ' + dir)
