import socket
import xml.dom.minidom
from xml_parsers import ClientParser
from juke_socket import ClientSocket

def getDirs(filelist):
   dirs = []
   dirxmls = filelist.getElementsByTagName('directory')
   for xml in dirxmls:
      dirs.append(xml.childNodes[0].data)
   return dirs

def getFiles(filelist):
   files = []
   filexmls = filelist.getElementsByTagName('file')
   for xml in filexmls:
      files.append(xml.childNodes[0].data)
   return files

class CommandSender:
   def __init__(self, address, port):
      self.address = address
      self.port = port
      self.socket = None
      self.xml = None

   def connect(self):
      try:
         self.socket = ClientSocket(self.address, self.port)
      except socket.error:
         raise SenderException('Cannot connect to server.')

   def send(self):
      if self.socket == None:
         raise RunTimeError, 'Forgot to connect the sender socket.'
      if self.xml == None:
         raise RunTimeError, 'Subclass must build message and put in self.xml.'
      self.socket.sendAll(self.xml)
      self.socket.shutdown(socket.SHUT_WR)
      retval = self.parseResponse(self.socket.receiveAll())
      self.socket.close()
      return retval

   def parseResponse(self, xmlStr):
      reply = xml.dom.minidom.parseString(xmlStr)
      pidNode = reply.getElementsByTagName('pid')[0]
      pid = pidNode.childNodes[0].data
      return int(pid)

class PlaySender(CommandSender):
   def __init__(self, filename, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='play'><filename>" + filename + "</filename></command>"

class PauseSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='pause'></command>"

class ResumeSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='resume'></command>"

class SkipFwdSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='skip-fwd'></command>"

class SkipBwdSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='skip-bwd'></command>"

class LsSender(CommandSender):
   def __init__(self, path, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='ls'><path>" + path + "</path></command>"

   def parseResponse(self, xmlStr):
      try:
         reply = xml.dom.minidom.parseString(xmlStr)
      except Exception as e:
         reply = 'error'
         print e
         print xmlStr
      filelist = reply.getElementsByTagName('filelist')[0]
      dirs = getDirs(filelist)
      files = getFiles(filelist)
      return (dirs, files)

class VolUpSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='vol-up'></command>"

class VolDownSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='vol-down'></command>"

class ExitSender(CommandSender):
   def __init__(self, address, port):
      CommandSender.__init__(self, address, port)
      self.xml = "<command type='exit'></command>"

class SenderException(Exception):
   def __init__(self, value):
      self.parameter = value

   def __str__(self):
      return str(self.parameter)
      #return repr(self.parameter)
