import xml.dom.minidom
from xml_parsers import ServerParser, XMLParser
from commands import *
from path_converter import PathConverter

class ServerMsgParser(XMLParser):
   def __init__(self, settings):
      self.settings = settings

   def load(self, xmlStr):
      if not self.settings.loaded:
         self.settings.load()
      self.converter = PathConverter(self.settings.getDirList())
      msg = xml.dom.minidom.parseString(xmlStr)
      self.storeCommand(msg)
      self.loaded = True

   def storeCommand(self, msg):
      commandNode = msg.getElementsByTagName('command')[0]
      type = commandNode.getAttribute('type')
      self.command = {
         'play': self.createPlayCommand,
         'pause': self.createPauseCommand,
         'resume': self.createResumeCommand,
         'skip-fwd': self.createSkipFwdCommand,
         'skip-bwd': self.createSkipBwdCommand,
         'vol-up': self.createVolUpCommand,
         'vol-down': self.createVolDownCommand,
         'exit': self.createExitCommand,
         'ls': self.createLsCommand
      }[type](commandNode)

   def createPlayCommand(self, command):
      cformat = self.settings.getPlay()
      fileNode = command.getElementsByTagName('filename')[0]
      filename = fileNode.childNodes[0].data
      filename = self.converter.virtualToAbsolute(filename)
      return PlayCommand(filename, cformat)

   def createPauseCommand(self, command):
      return PauseCommand(self.settings.getPause())

   def createPauseCommand(self, command):
      return PauseCommand(self.settings.getPause())

   def createResumeCommand(self, command):
      return PauseCommand(self.settings.getResume())

   def createSkipFwdCommand(self, command):
      return PauseCommand(self.settings.getSkipFwd())

   def createSkipBwdCommand(self, command):
      return PauseCommand(self.settings.getSkipBwd())

   def createVolUpCommand(self, command):
      return VolUpCommand(self.settings.getVolUp())

   def createVolDownCommand(self, command):
      return VolDownCommand(self.settings.getVolDown())

   def createExitCommand(self, command):
      return ExitCommand(self.settings.getExit())

   def createLsCommand(self, command):
      pathNode = command.getElementsByTagName('path')[0]
      vpath = pathNode.childNodes[0].data
      return LsCommand(self.settings.getDirList(), vpath)

   def getCommand(self):
      if self.loaded:
         return self.command

if __name__ == '__main__':
   msgparser = ServerMsgParser(ServerParser())
   msg = """ <message>
   <command type='play'><filename>/Main Videos/Shows/How I Met Your Mother/Season 1/01 Pilot.avi</filename>
   </command></message>"""
   msgparser.load(msg)
   command = msgparser.getCommand()
   command.execute()
