import xml.dom.minidom
from xml_parsers import XMLParser

class ClientEditor(XMLParser):
   def __init__(self):
      XMLParser.__init__(self)

   def loadFile(self, path):
      self.path = path

   def writeXML(self, dom):
      f = open(self.path, 'w')
      dom.writexml(f)
      f.close()

   def addSavedConnection(self, hostname, portnum):
      dom = xml.dom.minidom.parse(self.path)
      savedConns = dom.getElementsByTagName('client')[0].getElementsByTagName('saved-connections')[0]
      self.formSavedConnection(dom, savedConns, hostname, portnum)
      self.writeXML(dom)
      dom.unlink()

   def deleteOneSavedConnection(self, delhost, delport):
      dom = xml.dom.minidom.parse(self.path)
      savedConns = dom.getElementsByTagName('client')[0].getElementsByTagName('saved-connections')[0]
      self.removeSavedConnection(dom, savedConns, delhost, delport)
      self.writeXML(dom)
      dom.unlink()

   def removeSavedConnection(self, dom, savedConns, delhost, delport):
      for conn in savedConns.getElementsByTagName('connection'):
         if self.connEquals(conn, delhost, delport):
            savedConns.removeChild(conn)
            break # only remove it once if appears multiple times
      return savedConns

   def connEquals(self, conn, delhost, delport):
      hostNode = conn.getElementsByTagName('host')[0]
      hostName = hostNode.childNodes[0].data
      portNode = conn.getElementsByTagName('port')[0]
      portNum = portNode.childNodes[0].data
      if hostName == delhost and portNum == delport:
         return True
      else:
         return False

   def formSavedConnection(self, doc, savedConns, hostname, portnum):
      conn = doc.createElement('connection')
      savedConns.appendChild(conn)
      conn.appendChild(self.formHostNode(hostname, doc))
      conn.appendChild(self.formPortNode(portnum, doc))

   def formHostNode(self, hostname, doc):
      host = doc.createElement('host')
      hosttext = doc.createTextNode(hostname)
      host.appendChild(hosttext)
      return host

   def formPortNode(self, portnum, doc):
      port = doc.createElement('port')
      porttext = doc.createTextNode(portnum)
      port.appendChild(porttext)
      return port

   def writeLastConnection(self, host, port):
      dom = xml.dom.minidom.parse(self.path)
      lastConn = dom.getElementsByTagName('client')[0].getElementsByTagName('last-connection')[0]
      hostNode = lastConn.getElementsByTagName('host')[0]
      portNode = lastConn.getElementsByTagName('port')[0]
      lastConn.replaceChild(self.formHostNode(host, dom), hostNode)
      lastConn.replaceChild(self.formPortNode(str(port), dom), portNode)
      self.writeXML(dom)
      dom.unlink()
