import xml.dom.minidom
import os
from shutil import copyfile
from os import makedirs
from command_stores import *

mainConf = './juke_config.xml'
homeConf = os.getenv('HOME')+'/.config/juketv/juke_config.xml'
configDir = os.getenv('HOME')+'/.config/juketv'

class XMLParser:
   def __init__(self):
      self.loaded = False

   def load(self):
      if not os.path.exists(homeConf):
         try:
            try:
               makedirs(configDir)
            except:
               pass
            copyfile(mainConf, homeConf)
            self.loadFile(homeConf)
         except:
            print 'Error copying config file to home directory.'
            print 'Using the system wide configuration file.  This may cause'
            print 'problems saving connections if the user does not have write'
            print 'permissions on this file.'
            self.loadFile(mainConf)
      else:
         self.loadFile(homeConf)
      self.loaded = True

   def storeNetwork(self, network):
      portNode = network.getElementsByTagName('port')[0]
      self.port = portNode.childNodes[0].data
      addrNode = network.getElementsByTagName('server-address')[0]
      self.address = addrNode.childNodes[0].data

   def getPort(self):
      if self.loaded:
         return self.port

   def getAddress(self):
      if self.loaded:
         return self.address

class ServerParser(XMLParser):
   def __init__(self):
      XMLParser.__init__(self)

   def loadFile(self, path):
      dom = xml.dom.minidom.parse(path)
      server = dom.getElementsByTagName('server')[0]
      self.storeServerMaxAttempts(server.getElementsByTagName('max-reconnect-attempts')[0])
      self.storeServerLibrary(server.getElementsByTagName('library')[0])
      self.storeServerPlayback(server.getElementsByTagName('playback')[0])
      self.storeNetwork(dom.getElementsByTagName('network')[0])
      dom.unlink()

   def storeServerMaxAttempts(self, maxAts):
      self.maxAttempts = maxAts.childNodes[0].data

   def storeServerLibrary(self, library):
      self.storeRootDirs(library.getElementsByTagName('rootdir'))

   def storeRootDirs(self, rootDirs):
      self.dirList = []
      for rootDir in rootDirs:
         self.storeRootDir(rootDir)

   def storeRootDir(self, rootDir):
      nameNode = rootDir.getElementsByTagName('name')[0]
      name = nameNode.childNodes[0].data
      pathNode = rootDir.getElementsByTagName('path')[0]
      path = pathNode.childNodes[0].data
      self.dirList.append((name, path))  # (name, path) pairs

   def storeServerPlayback(self, playback):
      player = playback.getAttribute('player')

      if player == 'custom' or player == '':
         self.commands = CustomCommandStore(playback.getElementsByTagName('commands')[0])

   def getMaxAttempts(self):
      return int(self.maxAttempts)

   def getDirList(self):
      if self.loaded:
         return self.dirList

   def getPlay(self):
      if self.loaded:
         return self.commands.getPlay()

   def getPause(self):
      if self.loaded:
         return self.commands.getPause()

   def getResume(self):
      if self.loaded:
         return self.commands.getResume()

   def getSkipFwd(self):
      if self.loaded:
         return self.commands.getSkipFwd()

   def getSkipBwd(self):
      if self.loaded:
         return self.commands.getSkipBwd()

   def getVolUp(self):
      if self.loaded:
         return self.commands.getVolUp()

   def getVolDown(self):
      if self.loaded:
         return self.commands.getVolDown()

   def getExit(self):
      if self.loaded:
         return self.commands.getExit()

class ClientParser(XMLParser):
   def __init__(self):
      XMLParser.__init__(self)
      self.connList = []
      self.lastHost = ''
      self.lastPort = '51787'

   def loadFile(self, path):
      dom = xml.dom.minidom.parse(path)
      self.storeNetwork(dom.getElementsByTagName('network')[0])
      self.storeClientSpecific(dom.getElementsByTagName('client')[0])
      dom.unlink()

   def storeClientSpecific(self, client):
      try:
         self.storeLastConnection(client.getElementsByTagName('last-connection')[0])
      except:
         print 'Error loading last connection.  Continuing without...'
      try:
         self.storeSavedConnections(client.getElementsByTagName('saved-connections')[0])
      except:
         print 'Error loading saved connections.  Continuing without them...'

   def storeLastConnection(self, last_conns):
      self.lastHost = last_conns.getElementsByTagName('host')[0].childNodes[0].data
      self.lastPort = last_conns.getElementsByTagName('port')[0].childNodes[0].data

   def storeSavedConnections(self, saved_conns):
      connections = saved_conns.getElementsByTagName('connection')
      for conn in connections:
         self.storeSavedConnection(conn)

   def storeSavedConnection(self, connection):
      hostNode = connection.getElementsByTagName('host')[0]
      hostname = hostNode.childNodes[0].data
      portNode = connection.getElementsByTagName('port')[0]
      portnum = portNode.childNodes[0].data
      self.connList.append((hostname, portnum))

   def getConnectionList(self):
      return self.connList

   def getLastHost(self):
      return self.lastHost

   def getLastPort(self):
      return self.lastPort
